import { useEffect, useState } from "preact/hooks";
import { CheckboxInput, DefaultSeparator, FilterButton, LocationCard } from "../../components";
import { useClick, useFloating, useInteractions, Side, AlignedPlacement, flip, shift, useDismiss } from "@floating-ui/react";
import { JSXInternal } from "node_modules/preact/src/jsx";
import { getListRecentLocationsRatingsService, getRegenciesService, } from "../../services";
import { LocationInfo, Regency } from "../../domains";
import { ChangeEvent, TargetedEvent } from "preact/compat";
import { LocationType } from "../../types/common";
import { enumKeys } from "../../utils";
import { useNavigate } from "react-router-dom";
import './style.css';


interface floatFilterArgs<T> {
  isOpen: boolean,
  placement: Side | AlignedPlacement,
  outsidePress?: () => void,
  onChecked: (e: TargetedEvent<HTMLInputElement, Event>, id: number) => void,
  onClearFilter?: () => void,
  onInputChange: (val: string) => void,
  listData?: T[];
}

function FloatFilter(props: floatFilterArgs<Regency & { isSelected?: boolean }>) {
  const [floatState, setFloatState] = useState({
    searchValue: "",
  })

  const { refs, floatingStyles, context } = useFloating({
    placement: props.placement,
    open: props.isOpen,
    middleware: [
      flip({ fallbackAxisSideDirection: "end" }),
      shift()
    ]
  });

  const click = useClick(context);
  const dismiss = useDismiss(context, {
    outsidePress: () => {
      props.outsidePress && props.outsidePress()
      return false
    }
  });

  const { getReferenceProps, getFloatingProps } = useInteractions([
    click,
    dismiss
  ]);

  const style: JSXInternal.CSSProperties = {
    ...floatingStyles,
    top: 210,
    left: 'none',
    marginLeft: 150,
    width: 355,
    zIndex: 1,
  }

  function onInputChange(e: ChangeEvent<HTMLInputElement>): void {
    const input = e.target as HTMLInputElement;
    setFloatState({ searchValue: input.value.toLowerCase() })
    props.onInputChange(input.value)
  }

  if (props.isOpen) return (
    <div
      ref={refs.setFloating} {...getReferenceProps()}
      style={style}
      className={'bg-secondary float-filter-scroll'}
      {...getFloatingProps()}
    >
      <div style={{ padding: 10 }}>
        <input
          type="text"
          value={floatState.searchValue}
          onChange={onInputChange}
          placeholder={"Search ..."}
          className={'bg-quartenary text-sm input-text mb-3'}
        />
      </div>
      <div style={{ overflowY: 'scroll', height: 315, padding: 10 }} className={'float-filter-scroll'}>
        {props.listData ?
          <ul>
            {props.listData.map(x => (
              <div style={{ display: 'flex', alignItems: 'center', marginBottom: 5 }}>
                <input
                  type={"checkbox"}
                  id={x.id.toString()}
                  checked={x.isSelected}
                  value={x.id}
                  name={x.regency_name}
                  className={'input-checkbox'}
                  style={{ marginRight: 10, width: 18, height: 18 }}
                  onChange={(e) => props.onChecked(e, x.id)}
                />
                <label htmlFor={x.id.toString()}>{x.regency_name}</label>
              </div>
            ))}
          </ul>
          : null}
      </div>
      <div style={{ display: 'flex', justifyContent: 'space-around', paddingBottom: 10, paddingTop: 10 }}>
        <button onClick={props.onClearFilter}>Clear</button>
        <button onClick={props.outsidePress} >Close</button>
      </div>
    </div>
  )

  return null
}


function Discovery() {

  interface DiscoveryState {
    filterQ: string,
    regencies: any[],
    searchRegencies: any[],
    locations: LocationInfo[],
    locationType: Array<{value: string, isSelected?: boolean}>
  }

  const [data, setData] = useState<DiscoveryState>({
    filterQ: '',
    regencies: [],
    searchRegencies: [],
    locations: [],
    locationType: []
  });
  const [isFloatFilterOpen, setFloatFilterOpen] = useState(false)
  const navigate = useNavigate()

  useEffect(() => {
    getLocationType()
    getRecentLocations()
    getRegion()
  }, [])

  async function getRegion() {
    try {
      const res = await getRegenciesService();
      setData((prevState) => ({ ...prevState, regencies: res.data, searchRegencies: res.data }))
    } catch (err) {
      alert(err)
    }
  }

  async function getRecentLocations() {
    try {
      const locations = await getListRecentLocationsRatingsService(15, 1)
      setData((prevState) => ({ ...prevState, locations: locations.data }))
    } catch (error) {
      console.log(error)
    }
  }

  function onCheckedRegencyFilter(_val: any, id: number) {
    // const value = val as HTMLInputElement;
    const dataRegencies = data.searchRegencies as (Regency & { isSelected?: boolean })[];
    const regencyIdx = dataRegencies.findIndex(x => x.id == id);
    dataRegencies[regencyIdx].isSelected = !dataRegencies[regencyIdx].isSelected
    setData({
      ...data,
      regencies: dataRegencies,
      searchRegencies: dataRegencies
    })
  }

  function onClearFilter() {
    const dataRegencies = data.searchRegencies as (Regency & { isSelected?: boolean })[];
    const regencies = dataRegencies.map(x => ({ ...x, isSelected: false }))

    setData({
      ...data,
      regencies: regencies,
      searchRegencies: regencies
    })
  }


  function onInputChange(search: string) {
    const dataRegencies = data.regencies as (Regency & { isSelected?: boolean })[];
    // console.log(dataRegencies.filter(x => x.regency_name.toLowerCase().includes(search)))
    setData({
      ...data,
      searchRegencies: dataRegencies.filter(x => x.regency_name.toLowerCase().includes(search))
    })
  }


  function onClickedTypeLocations(idx: number) {
    const locType = data.locationType
    locType[idx].isSelected = !locType[idx].isSelected
    setData({
      ...data,
      locationType: locType
    })
  }

  function onApplyFilter() {
    const dataRegencies = data.regencies as (Regency & { isSelected?: boolean})[];
    const selectedRegencies = dataRegencies.filter(x => x.isSelected)
    const selectedLocType = data.locationType.filter(x => x.isSelected)
    let regenciesQ = "";
    let locTypeQ = ""

    if(selectedRegencies.length > 0) {
      regenciesQ = selectedRegencies.map(x => x.id).join(" OR ").replace(/\s+/g, '%20')
    
    }
    if(selectedLocType.length > 0) {
      const onJoin = selectedRegencies.length > 0 ? " AND " : " OR "
      locTypeQ = selectedLocType.map(x => x.value).join(onJoin).replace(/\s+/g, '%20')

      if(selectedRegencies.length > 0) locTypeQ = `%20AND%20${locTypeQ}`
    }

    const searchQ = `${regenciesQ}${locTypeQ}`

    


  }

  function getLocationType() {
    const type: Array<{value: string, isSelected?: boolean}> = []
    for (const lt of enumKeys(LocationType)) {
      type.push({value: LocationType[lt]});
    }

    setData((prevState) => ({ ...prevState, locationType: type }))

  }

  function onClickLocation(id: number) {
    navigate(`/location/${id}`)
  }

  function onDeleteByCityFilter(value: any) {
    const dataRegencies = data.searchRegencies as (Regency & { isSelected?: boolean })[];
    const regencyIdx = dataRegencies.findIndex(x => x.id == value.id);
    dataRegencies[regencyIdx].isSelected = !dataRegencies[regencyIdx].isSelected
    setData({
      ...data,
      regencies: dataRegencies,
      searchRegencies: dataRegencies
    })
  }

  return (
    <div className="content main-content mt-3">
      <section name="header">
        <h1 className={'text-3xl mb-5 font-bold'}>Discovery</h1>
        <DefaultSeparator />
      </section>
      <section name="content">
        <div style="padding-bottom: 24px; justify-content: flex-start; align-items: flex-start; display: inline-flex">
          {/* FILTER */}
          <section name="discovery filter">
            <div className="filter-container">
              <div style="align-self: stretch; padding-left: 12px; padding-right: 12px; padding-top: 8px; padding-bottom: 8px; background: #36383B; border-radius: 8px; justify-content: flex-start; align-items: center; gap: 10px; display: inline-flex">
                <div style="flex: 1 1 0; color: white; font-size: 16px; line-height: 24px; letter-spacing: 0.64px; word-wrap: break-word">Filter</div>
              </div>
              <div style="align-self: stretch; padding-left: 12px; padding-right: 12px; flex-direction: column; justify-content: flex-start; align-items: flex-start; gap: 16px; display: flex">
                <div style="align-self: stretch; color: white; font-size: 14px; font-weight: 700; line-height: 21px; letter-spacing: 0.56px; word-wrap: break-word">Kota / Kabupaten </div>
                <div style="align-self: stretch; flex-direction: column; justify-content: flex-start; align-items: flex-start; gap: 12px; display: flex">
                  {data.regencies.filter(x => x.isSelected).map(x => (
                    <div style="align-self: stretch; justify-content: flex-start; align-items: center; gap: 12px; display: inline-flex">
                      <div style="flex: 1 1 0; color: white; font-size: 14px; font-family: Poppins; font-weight: 500; line-height: 21px; letter-spacing: 0.56px; word-wrap: break-word">{x.regency_name}</div>
                      <button onClick={() => onDeleteByCityFilter(x)} style={{ color: "white"}}>x</button>
                    </div>
                  ))}
                </div>
                <div style="align-self: stretch; padding: 8px; border-radius: 6px; justify-content: center; align-items: center; gap: 4px; display: inline-flex">
                  <button onClick={() => setFloatFilterOpen(!isFloatFilterOpen)} style="text-align: center; color: #85CE73; font-size: 14px; font-weight: 600; line-height: 21px; letter-spacing: 0.10px; word-wrap: break-word">Lihat Selengkapnya</button>
                  <FloatFilter
                    isOpen={isFloatFilterOpen}
                    placement="right"
                    listData={data.searchRegencies}
                    outsidePress={() => setFloatFilterOpen(false)}
                    onClearFilter={onClearFilter}
                    onChecked={onCheckedRegencyFilter}
                    onInputChange={onInputChange}
                  />
                </div>
              </div>
              <div style="align-self: stretch; height: 2px; background: #36383B; border-radius: 2px"></div>
              <div style="align-self: stretch; height: 190px; padding-left: 12px; padding-right: 12px; flex-direction: column; justify-content: flex-start; align-items: flex-start; gap: 16px; display: flex">
                <div style="align-self: stretch; color: white; font-size: 14px; font-family: Poppins; font-weight: 700; line-height: 21px; letter-spacing: 0.56px; word-wrap: break-word">Tipe</div>
                <div style="align-self: stretch; height: 153px; flex-direction: column; justify-content: flex-start; align-items: flex-start; gap: 12px; display: flex">
                  {data.locationType.map((x, i) => (
                    <CheckboxInput
                      inputName="location_type"
                      onChange={() => onClickedTypeLocations(i)}
                      value={x.value}
                      isSelected={x.isSelected}
                    />
                  ))}
                </div>
              </div>
              <div style="align-self: stretch; height: 2px; background: #36383B; border-radius: 2px"></div>
              <FilterButton onClick={onApplyFilter} label="Apply" />
            </div>
          </section>
          {/*  FILTER END */}
          <div>
            {data.locations.map(x => (
              <LocationCard
                containerClass='card-list-container'
                onCardClick={onClickLocation}
                data={x}
              />
            ))}
          </div>
        </div>
      </section>
    </div>
  )

}

export default Discovery;